-- $Id: ClientInterceptor.lua 92963 2009-06-26 02:43:09Z augusto $

local oil = require "oil"
local orb = oil.orb

local oop = require "loop.base"

local log = require "openbus.util.Log"

---
--Interceptador de requisies de servio, responsvel por inserir no contexto
--da requisio a credencial do cliente.
---
module("openbus.interceptors.ClientInterceptor", oop.class)

---
--Cria o interceptador.
--
--@param config As configuraes do interceptador.
--@param credentialManager O objeto onde a credencial do membro fica armazenada.
--
--@return O interceptador.
---
function __init(self, config, credentialManager)

  log:interceptor("Construindo interceptador para cliente")
  local lir = orb:getLIR()
  return oop.rawnew(self,
                    {credentialManager = credentialManager,
                     credentialType = lir:lookup_id(config.credential_type).type,
                     contextID = config.contextID})
end

---
--Intercepta o request para envio da informao de contexto (credencial)
--
--@param request Informaes sobre a requisio.
---
function sendrequest(self, request)
  log:interceptor("INTERCEPTAO CLIENTE OP: "..request.operation)

  -- Verifica de existe credencial para envio
  if not self.credentialManager:hasValue() then
    log:interceptor "SEM CREDENCIAL !"
    return
  end
  log:interceptor("TEM CREDENCIAL!")

  -- Insere a credencial no contexto do servio
  local encoder = orb:newencoder()
  encoder:put(self.credentialManager:getValue(),
              self.credentialType)
  request.service_context =  {
    { context_id = self.contextID, context_data = encoder:getdata() }
  }
  log:interceptor("INSERI CREDENCIAL")
end
